/* mainCSA.cpp
   Copyright 2003, V. Makinen, all rights reserved.

   An interface to CSA.cpp.

   This software may be used freely for any purpose. However, when distributed,
   the original source must be clearly stated, and, when the source code is
   distributed, the copyright notice must be retained and any alterations in
   the code must be clearly marked. No warranty is given regarding the quality
   of this software.
*/

#include	<stdio.h>
#include	<stdlib.h>
#include	"386.c"
#include        "CSA.cpp"
#include	<string.h>
#include        <unistd.h>
#include        <fcntl.h>
#include        <iostream.h>

char buf[720000000];
char *wbuf;

extern int hz;
extern char *filename;
extern unsigned C,D;
extern unsigned savemem;
extern bool heuristic;

main(int argc, char *argv[])
{
	int c;
	extern char *optarg;
	extern int optind;

	int nexec = 1;
        int alg = 0;
        int i;

	long tp, te, nf, ti;
	int nw;
	int nbuf, len;
	char *s;	

	while((c = getopt(argc, argv, "e:a:p:r:C:D:s:")) != -1)
		switch(c)
		{
		case 'e':	nexec = atoi(optarg); break;
                case 'a':	alg = atoi(optarg); break;
                case 'p':	print = atoi(optarg); break;
		case 'r':	report = atoi(optarg); break;
                case 'C':	C = atoi(optarg); break;
		case 'D':	D = atoi(optarg); break;
		case 's':	savemem = atoi(optarg); break;
		default:	goto usage;
		}
	if((optind >= argc) || (optind+2 < argc)){
usage:
		fprintf(stderr, 
		"usage: %s [-a alg] [-e nex] [-p print] [-r report] [-C length] [-D depth] [-s savemem] [string] file\n", argv[0]);
		
		fprintf(stderr,"alg = {0=SearchCSA,1=SearchSA,2=SearchCSALCP,3=SearchSALCP,\n");
		fprintf(stderr,"       4=Uncompact,5=SearchCSALCPfit}\n");
		fprintf(stderr,"nex = number of execution times\n");
		fprintf(stderr,"print = {0,1}\n");
		fprintf(stderr,"existence,report,count = {0,1,2}\n");
		fprintf(stderr,"length = maximum length of the linked areas\n");
		fprintf(stderr,"depth = maximum length of the link paths\n");	
		fprintf(stderr,"savemem = {0,1}\n");				
		exit(1);
	}
	if((c = open(argv[argc-1], 0)) < 0){
		perror(argv[argc-1]);
		exit(1);
	}
	filename = (char *)malloc(sizeof(char)*(strlen(argv[argc-1])+1));

    strcpy(filename,argv[argc-1]);
	nbuf = read(c, buf+1, 100000000*sizeof(char));
	buf[++nbuf] = '#';
	buf[nbuf+1] = '\0';
	close(c);
	text = buf+1;
	n = nbuf;
	if(optind+2 == argc){
		len = strlen(argv[optind]);
                wbuf = (char *)malloc((len+3)*sizeof(char));
		strcpy(wbuf, argv[optind]);
		wbuf[len] = '\0';
		wbuf[len+1] = '\0';
	} else {
                wbuf = (char*)malloc(12000000*sizeof(char));
                i = 0;
		while((c = getchar()) != EOF) {
			wbuf[i++] = (c=='\n')? '\0':c;
                }
		wbuf[i++] = '\0'; wbuf[i] = '\0';
	}
	tp = te = ti = 0;
	nf = nw = 0;
        startclock();
        if (alg !=7)
           MakeSuffixArray(alg);
        ti += stopclock();
        startclock();

   	for(s = wbuf; s[0] != '\0';) {
     		nw++;
		len = strlen(s);
		m = len;
		for(c = 0; c < nexec; c++)
		   if (alg == 0 && report != 2)
   		      nf += SearchFromCSA(s,CSA,SuffixArrayCompactSize);
		   else if (alg == 0)
   		      nf += SearchFromCSACount(s,CSA,SuffixArrayCompactSize);
   		   else if (alg == 1 && report != 2)
   		      nf += SearchFromSA(s,SA,n);
		   else if (alg ==1)
   		      nf += SearchFromSACount(s,SA,n);
   		   else if (alg == 2)
   		      nf += SearchFromCSAWithLCP(s,CSA,SuffixArrayCompactSize);
   		   else if (alg == 3)
   		      nf += SearchFromSAWithLCP(s,SA,n);
   		   else if (alg == 4)
   		      UnCompactSuffixArray();
   		   else if (alg == 5)
   		      nf += SearchFromCSAWithLCP(s,CSA,SuffixArrayCompactSize);
   		if (alg == 4)
   		   nf = 0;
		s += len;
		/* skip a possible empty word */
		if (s[0] == '\0') s++;
   	   }
           te += stopclock();

      	   printf("alg=%s a=%d w=%d match=%ld \n i=%4fs e=%.4fs(%ldms/%d)\n",
                argv[0], alg, nw, (long)nf/nexec,
                (ti*1.0)/(hz*1.0),
		(te*1.0)/(hz*nexec),
		te, nexec);
           tp = 0;
           te = 0;
           nf = 0;
           nw = 0;
	return 0;
}

